# Topology Digital Substation 
# !/usr/bin/python

from mininet.net import Mininet
from mininet.node import Node, Controller, RemoteController, OVSSwitch, OVSKernelSwitch, Host
from mininet.cli import CLI
from mininet.link import Intf, TCLink
from mininet.log import setLogLevel, info
from mininet.node import Node, CPULimitedHost
from mininet.util import irange, dumpNodeConnections
import time
import os


class LinuxRouter(Node):
    "A Node with IP forwarding enabled."

    def config(self, **params):
        super(LinuxRouter, self).config(**params)
        # Enable forwarding on the router
        self.cmd('sysctl net.ipv4.ip_forward=1')

    def terminate(self):
        self.cmd('sysctl net.ipv4.ip_forward=0')
        super(LinuxRouter, self).terminate()


def emptyNet():
    NODE2_IP = '192.168.56.1'
    CONTROLLER_IP = '127.0.0.1'

    net = Mininet(topo=None,
                  build=False)

    # c0 = net.addController( 'c0',controller=RemoteController,ip=CONTROLLER_IP,port=6633)
    net.addController('c0', port=6633)

    r0 = net.addHost('r0', cls=LinuxRouter, ip='100.0.0.1/16')
    r7 = net.addHost('r7', cls=LinuxRouter, ip='100.7.0.1/16')

    # Switch External Gateway
    s777 = net.addSwitch('s777')

    # Switch on Control Center
    s999 = net.addSwitch('s999')

    # Switch on Substation
    s71 = net.addSwitch('s71')
    s72 = net.addSwitch('s72')
    s73 = net.addSwitch('s73')

    # Add host-switch links in the same subnet
    net.addLink(s999, r0, intfName2='r0-eth1', params2={'ip': '100.0.0.1/16'})
    net.addLink(s71, r7, intfName2='r7-eth1', params2={'ip': '100.7.0.1/16'})

    # Add router-router link in a new subnet for the router-router connection
    net.addLink(r0, r7, intfName1='r0-eth3', intfName2='r7-eth2', params1={'ip': '200.7.0.1/24'},
                params2={'ip': '200.7.0.2/24'})

    # Add Host on Control Center
    ccdb = net.addHost('ccdb', ip='100.0.0.11')
    operator = net.addHost('operator', ip='100.0.0.12')
    hacker = net.addHost('hacker', ip='100.0.0.13')

    # Add Hosts on Substation 7
    s07m1 = net.addHost('s07m1', ip='100.7.0.11', cls=CPULimitedHost, cpu=.1)
    s07m2 = net.addHost('s07m2', ip='100.7.0.12', cls=CPULimitedHost, cpu=.1)
    s07m3 = net.addHost('s07m3', ip='100.7.0.13', cls=CPULimitedHost, cpu=.1)
    s07m4 = net.addHost('s07m4', ip='100.7.0.14', cls=CPULimitedHost, cpu=.1)
    s07m5 = net.addHost('s07m5', ip='100.7.0.15', cls=CPULimitedHost, cpu=.1)
    s07m6 = net.addHost('s07m6', ip='100.7.0.16', cls=CPULimitedHost, cpu=.1)
    s07m7 = net.addHost('s07m7', ip='100.7.0.17', cls=CPULimitedHost, cpu=.1)
    s07m8 = net.addHost('s07m8', ip='100.7.0.18', cls=CPULimitedHost, cpu=.1)
    s07m9 = net.addHost('s07m9', ip='100.7.0.19', cls=CPULimitedHost, cpu=.1)
    s07m10 = net.addHost('s07m10', ip='100.7.0.20', cls=CPULimitedHost, cpu=.1)
    s07m13 = net.addHost('s07m13', ip='100.7.0.30', cls=CPULimitedHost, cpu=.1)
    s07m14 = net.addHost('s07m14', ip='100.7.0.40', cls=CPULimitedHost, cpu=.1)
    s07m15 = net.addHost('s07m15', ip='100.7.0.50', cls=CPULimitedHost, cpu=.1)
    s07m16 = net.addHost('s07m16', ip='100.7.0.60', cls=CPULimitedHost, cpu=.1)
    s07m17 = net.addHost('s07m17', ip='100.7.0.70', cls=CPULimitedHost, cpu=.1)
    s07m18 = net.addHost('s07m18', ip='100.7.0.80', cls=CPULimitedHost, cpu=.1)
    s07m19 = net.addHost('s07m19', ip='100.7.0.90', cls=CPULimitedHost, cpu=.1)
    s07cpc = net.addHost('s07cpc', ip='100.7.0.21')
    s07db = net.addHost('s07db', ip='100.7.0.22')
    s07gw = net.addHost('s07gw', ip='100.7.0.23')

    # Link siwtch to switch
    net.addLink(s71, s72)
    net.addLink(s73, s72)

    # Link Control Center to Switch
    net.addLink(ccdb, s999, intfName1='ccdb-eth1', params1={'ip': '100.0.0.11/24'})
    net.addLink(operator, s999, intfName1='operator-eth1', params1={'ip': '100.0.0.12/24'})
    net.addLink(hacker, s999, intfName1='hacker-eth1', params1={'ip': '100.0.0.13/24'})

    # Link Substation 07 Merging unit to Switch
    net.addLink(s07m1, s73, intfName1='s07m1-eth1', params1={'ip': '100.7.0.11/24'})
    net.addLink(s07m2, s73, intfName1='s07m2-eth1', params1={'ip': '100.7.0.12/24'})
    net.addLink(s07m3, s73, intfName1='s07m3-eth1', params1={'ip': '100.7.0.13/24'})
    net.addLink(s07m4, s73, intfName1='s07m4-eth1', params1={'ip': '100.7.0.14/24'})
    net.addLink(s07m5, s73, intfName1='s07m5-eth1', params1={'ip': '100.7.0.15/24'})
    net.addLink(s07m6, s73, intfName1='s07m6-eth1', params1={'ip': '100.7.0.16/24'})
    net.addLink(s07m7, s73, intfName1='s07m7-eth1', params1={'ip': '100.7.0.17/24'})
    net.addLink(s07m8, s73, intfName1='s07m8-eth1', params1={'ip': '100.7.0.18/24'})
    net.addLink(s07m9, s73, intfName1='s07m9-eth1', params1={'ip': '100.7.0.19/24'})
    net.addLink(s07m10, s73, intfName1='s07m10-eth1', params1={'ip': '100.7.0.20/24'})
    net.addLink(s07m13, s72, intfName1='s07m13-eth1', params1={'ip': '100.7.0.30/24'})
    net.addLink(s07m14, s72, intfName1='s07m14-eth1', params1={'ip': '100.7.0.40/24'})
    net.addLink(s07m15, s72, intfName1='s07m15-eth1', params1={'ip': '100.7.0.50/24'})
    net.addLink(s07m16, s72, intfName1='s07m16-eth1', params1={'ip': '100.7.0.60/24'})
    net.addLink(s07m17, s72, intfName1='s07m17-eth1', params1={'ip': '100.7.0.70/24'})
    net.addLink(s07m18, s72, intfName1='s07m18-eth1', params1={'ip': '100.7.0.80/24'})
    net.addLink(s07m19, s72, intfName1='s07m19-eth1', params1={'ip': '100.7.0.90/24'})
    net.addLink(s07cpc, s72)
    net.addLink(s07db, s72)
    net.addLink(s07gw, s71, intfName1='s07gw-eth1', params1={'ip': '100.7.0.23/24'})

    # Link Host Control Center to External gateway
    net.addLink(ccdb, s777, intfName1='ccdb-eth0', params1={'ip': '10.0.0.11/16'})
    net.addLink(operator, s777, intfName1='operator-eth0', params1={'ip': '10.0.0.12/16'})
    net.addLink(hacker, s777, intfName1='hacker-eth0', params1={'ip': '10.0.0.13/16'})

    # Link Host Substation 7 to switch to external gateway
    net.addLink(s07m1, s777, intfName1='s07m1-eth0', params1={'ip': '10.0.7.11/16'})
    net.addLink(s07m2, s777, intfName1='s07m2-eth0', params1={'ip': '10.0.7.12/16'})
    net.addLink(s07m3, s777, intfName1='s07m3-eth0', params1={'ip': '10.0.7.13/16'})
    net.addLink(s07m4, s777, intfName1='s07m4-eth0', params1={'ip': '10.0.7.14/16'})
    net.addLink(s07m5, s777, intfName1='s07m5-eth0', params1={'ip': '10.0.7.15/16'})
    net.addLink(s07m6, s777, intfName1='s07m6-eth0', params1={'ip': '10.0.7.16/16'})
    net.addLink(s07m7, s777, intfName1='s07m7-eth0', params1={'ip': '10.0.7.17/16'})
    net.addLink(s07m8, s777, intfName1='s07m8-eth0', params1={'ip': '10.0.7.18/16'})
    net.addLink(s07m9, s777, intfName1='s07m9-eth0', params1={'ip': '10.0.7.19/16'})
    net.addLink(s07m10, s777, intfName1='s07m10-eth0', params1={'ip': '10.0.7.20/16'})
    net.addLink(s07m13, s777, intfName1='s07m13-eth0', params1={'ip': '10.0.7.30/16'})
    net.addLink(s07m14, s777, intfName1='s07m14-eth0', params1={'ip': '10.0.7.40/16'})
    net.addLink(s07m15, s777, intfName1='s07m15-eth0', params1={'ip': '10.0.7.50/16'})
    net.addLink(s07m16, s777, intfName1='s07m16-eth0', params1={'ip': '10.0.7.60/16'})
    net.addLink(s07m17, s777, intfName1='s07m17-eth0', params1={'ip': '10.0.7.70/16'})
    net.addLink(s07m18, s777, intfName1='s07m18-eth0', params1={'ip': '10.0.7.80/16'})
    net.addLink(s07m19, s777, intfName1='s07m19-eth0', params1={'ip': '10.0.7.90/16'})
    net.addLink(s07gw, s777, intfName1='s07gw-eth0', params1={'ip': '10.0.7.23/16'})

    # Build and start Network 
    net.build()
    net.addNAT(ip='10.0.0.250').configDefault()
    net.start()

    # Configure Tunnel
    nat = net.get('nat0')
    nat.cmdPrint('ip link set mtu 1454 dev nat0-eth0')

    # Add routing for reaching networks that aren't directly connected
    info(net['r0'].cmd('ip route add 100.7.0.0/24 via 200.7.0.2 dev r0-eth3'))
    info(net['r7'].cmd('ip route add 100.0.0.0/24 via 200.7.0.1 dev r7-eth2'))

    info(net['s07m1'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m1-eth1'))
    info(net['s07m2'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m2-eth1'))
    info(net['s07m3'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m3-eth1'))
    info(net['s07m4'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m4-eth1'))
    info(net['s07m5'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m5-eth1'))
    info(net['s07m6'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m6-eth1'))
    info(net['s07m7'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m7-eth1'))
    info(net['s07m8'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m8-eth1'))
    info(net['s07m9'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m9-eth1'))
    info(net['s07m10'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m10-eth1'))
    info(net['s07m13'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m13-eth1'))
    info(net['s07m14'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m14-eth1'))
    info(net['s07m15'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m15-eth1'))
    info(net['s07m16'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m16-eth1'))
    info(net['s07m17'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m17-eth1'))
    info(net['s07m18'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m18-eth1'))
    info(net['s07m19'].cmd('ip route add 100.0.0.0/24 via 100.7.0.1 dev s07m19-eth1'))

    info(net['ccdb'].cmd('ip route add 100.7.0.0/24 via 100.0.0.1 dev ccdb-eth1'))
    info(net['operator'].cmd('ip route add 100.7.0.0/24 via 100.0.0.1 dev operator-eth1'))
    info(net['hacker'].cmd('ip route add 100.7.0.0/24 via 100.0.0.1 dev hacker-eth1'))

    info(os.system('ip addr add 100.0.0.99/24 dev s999'))
    info(os.system('ip link set s999 up'))

    CLI(net)
    net.stop()


if __name__ == '__main__':
    setLogLevel('info')
    emptyNet()
