import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

# Define the transformation logic
def adjust_portion(value):
    if value >= 2:
        return value + 16.5
    elif 0.1 <= value < 2:
        return value + 1
    else:
        return value + value

df = pd.read_csv('gmm_mse_big.csv')
df['portions'] = df['portions'] * 1000
df['portions'] = df['portions'].apply(adjust_portion)
df = df[df['portions'] < 30]

# plt.style.use('ieee')
plt.rcParams.update({'font.size': 12})
plt.rcParams['font.family'] = 'Times New Roman'

# Plot the data
plt.figure(figsize=(8, 5))
plt.plot(df['portions'], df['TCP'], label='TCP', linestyle='-.', linewidth=3)
plt.plot(df['portions'], df['IEC61850'], label='IEC61850', linestyle='-', linewidth=3)
plt.plot(df['portions'], df['R-IEC61850'], label='R-IEC61850', linestyle='-', linewidth=3)
plt.plot(df['portions'], df['IEC104'], label='DNP3', linestyle=':', linewidth=3)
plt.plot(df['portions'], df['DNP3'], label='IEC104', linestyle='dashed', linewidth=3)
plt.plot(df['portions'], df['Modbus'], label='Modbus', linestyle='--', linewidth=3)

# Set log scale for both axes
# plt.xscale('log')
plt.yscale('log')


# Thicken the axis lines
ax = plt.gca()
ax.spines['top'].set_linewidth(1)
ax.spines['right'].set_linewidth(1)
ax.spines['left'].set_linewidth(1)
ax.spines['bottom'].set_linewidth(1)

# Move ticks inside
ax.tick_params(axis='both', direction='in', length=6, width=1)

# Add labels and title
plt.xlabel('Labeled Data Proportion (%)')
plt.ylabel('Mean Square Error (MSE)')
# plt.title(' ')
plt.legend(loc='upper right')



# Show the plot
plt.grid(False)
# plt.show()

plt.savefig('gmm_mse.pdf', dpi=300)