import os
import numpy as np
from data_loader import extract_data, get_label
import PIL
from PIL import Image
from numpy import asarray
import binascii
import pandas as pd
import tensorflow as tf
from tensorflow.keras.layers import Dense, Input, Flatten, Conv2D, Dropout, MaxPooling2D
from tensorflow.keras import models, datasets, layers, optimizers
from tensorflow.keras import backend as k
import matplotlib.pyplot as plt
from sklearn.model_selection import train_test_split



THIS_FOLDER = os.path.dirname(os.path.abspath(__file__))

attack_folder = THIS_FOLDER+"\\data_IoT\\Attack"
normal_folder = THIS_FOLDER+"\\data_IoT\\Normal"

A_data, A_c = extract_data(attack_folder)
AN_data, AN_c = extract_data(normal_folder)

A_Y = get_label(AN_c, A_c)
A_X = np.concatenate((AN_data,A_data), axis=0)

# Generate shuffling indices based on the common dimension
shuffling_indices = np.arange(A_Y.shape[0])
np.random.shuffle(shuffling_indices)
A_Ys = np.take(A_Y, shuffling_indices, axis=0)
A_Xs = np.take(A_X, shuffling_indices, axis=0)

# Data preparation
x_train = A_Xs.reshape((-1,16,16,1))
num_category = 2


a, b, c, d = train_test_split(
    x_train, A_Ys, test_size=0.3, stratify=A_Ys, random_state=53)

#Build Model
IMG_SIZE = (16, 16, 1)
input_img = layers.Input(shape=IMG_SIZE)

model = layers.Conv2D(32, (3, 3), padding='same')(input_img)
model = layers.Activation('relu')(model)
model = layers.Conv2D(32, (3, 3), padding='same', strides=(2, 2))(model)
model = layers.Activation('relu')(model)

model = layers.Conv2D(32, (3, 3), padding='same')(model)
model = layers.Activation('relu')(model)
model = layers.Conv2D(32, (3, 3), padding='same', strides=(2, 2))(model)
model = layers.Activation('relu')(model)

model = layers.Conv2D(64, (3, 3), padding='same')(model)
model = layers.Activation('relu')(model)
model = layers.Conv2D(64, (3, 3), padding='same', strides=(2, 2))(model)
model = layers.Activation('relu')(model)

model = layers.Conv2D(128, (3, 3), padding='same')(model)
model = layers.Activation('relu')(model)
model = layers.Conv2D(128, (3, 3), padding='same', strides=(2, 2))(model)
model = layers.Activation('relu')(model)

model = layers.Conv2D(128, (3, 3), padding='same')(model)
model = layers.Activation('relu')(model)
model = layers.Conv2D(128, (3, 3), padding='same', strides=(2, 2))(model)
model = layers.Activation('relu')(model)

model = layers.Conv2D(64, (3, 3), padding='same')(model)
model = layers.Activation('relu')(model)
model = layers.Conv2D(64, (3, 3), padding='same')(model)
model = layers.Activation('relu')(model)

model = layers.Conv2D(32, (3, 3), padding='same')(model)
model = layers.Activation('relu')(model)
model = layers.Conv2D(32, (3, 3), padding='same', strides=(2, 2))(model)
model = layers.Activation('relu')(model)


model = layers.GlobalAveragePooling2D()(model)
model = layers.Dense(32)(model)
model = layers.Activation('relu')(model)
model = layers.Dense(1)(model)

output_img = layers.Activation('softmax')(model)

model = models.Model(input_img, output_img)

model.summary()

adam = optimizers.Adam(lr=0.0001)
model.compile(adam, loss='binary_crossentropy', metrics=["accuracy"])

history = model.fit(a, c, epochs=10,
                    validation_data=(b, d))



plt.rcParams["font.family"] = "Arial"
plt.rcParams["font.size"] = 8
plt.rcParams['pdf.fonttype'] = 42
plt.rc('xtick', labelsize=8)
plt.rc('ytick', labelsize=8)
plt.rc('axes', labelsize=8)


width = 6
height = 4
plt.rcParams["figure.figsize"] = (width,height)
plt.title('Learning Accuracy')

plt.plot(history.history['accuracy'], label='accuracy')
plt.plot(history.history['val_accuracy'], label='validation accuracy')
#plt.plot(history.history['loss'], label='loss')
plt.xlabel('Epoch')
plt.ylabel('Accuracy')
plt.ylim([0, 1])
plt.legend(loc='best')

test_loss, test_accuracy = model.evaluate(b, d, verbose=2)
print('\nTest accuracy = {0:.2f}%'.format(test_accuracy*100.0))

