# Dissertation Source Code Documentation
## **Advanced Persistent Threat Detection and Correlation for Cyber-Physical Power Systems**

This repository contains partial code, data, and outputs for each chapter of the dissertation. Below is an organized documentation of the structure and content of each directory.

## Folder Structure

```
Dissertation_data/
├── Chapter_2
│   └── Mininet_Model
│       └── mininet_network.py
├── Chapter_3
│   └── Diagram
│       └── ACPPS_Kill_Chain.pdf
├── Chapter_4
│   ├── 1_Input
│   │   ├── a7.csv
│   │   ├── a7.npy
│   │   ├── data_traffic_normal.csv
│   │   └── s7_traffic.csv
│   ├── 2_Code
│   │   ├── plot_traffic_histogram.py
│   │   └── plot_traffic_line.py
│   └── 3_Output
│       ├── attack_graph.png
│       ├── cps_model.jpg
│       ├── diagram.jpg
│       ├── traffic_histogram.png
│       └── traffic_line_plot.png
├── Chapter_5
│   ├── 1_Input
│   │   ├── Attack
│   │   │   └── malware.pcap
│   │   ├── Normal
│   │   │   └── normal.pcapng
│   │   └── gmm_mse_big.csv
│   ├── 2_Code
│   │   ├── plot_gmm_mse.py
│   │   └── train_CNN_calssifier.py
│   └── 3_Output
│       └── gmm_mse.pdf
├── Chapter_6
│   ├── 1_Input
│   │   ├── dataB.csv
│   │   ├── length_data_1.csv
│   │   ├── length_data_2n.csv
│   │   └── length_data_3.csv
│   ├── 2_Code
│   │   ├── box_plots.py
│   │   └── plot_scatter_2D.py
│   └── 3_Output
│       ├── scatter_plot2D.pdf
│       └── traffic_statistic.pdf
```

## Chapter-wise Documentation

### Chapter 2: Mininet Model
- **File:** `mininet_network.py`
  - Description: Python script for setting up a Mininet network model.

### Chapter 3: Diagram
- **File:** `ACPPS_Kill_Chain.pdf`
  - Description: A PDF document illustrating the ACPPS Kill Chain diagram.

### Chapter 4: Traffic Analysis
#### Input Files
- `a7.csv`, `a7.npy`: Data files for traffic analysis.
- `data_traffic_normal.csv`, `s7_traffic.csv`: CSV files containing traffic data.

#### Code Files
- `plot_traffic_histogram.py`: Generates histograms for traffic data.
- `plot_traffic_line.py`: Creates line plots for traffic data.

#### Output Files
- `attack_graph.png`, `cps_model.jpg`, `diagram.jpg`: Generated diagrams and models.
- `traffic_histogram.png`, `traffic_line_plot.png`: Visualizations of traffic data.

### Chapter 5: GMM and CNN Classifier
#### Input Files
- `Attack/malware.pcap`: PCAP file containing malware traffic.
- `Normal/normal.pcapng`: PCAP file containing normal traffic.
- `gmm_mse_big.csv`: CSV file for GMM analysis.

#### Code Files
- `plot_gmm_mse.py`: Script to visualize GMM MSE.
- `train_CNN_calssifier.py`: Code for training a CNN classifier.

#### Output Files
- `gmm_mse.pdf`: Output visualization of GMM MSE.

### Chapter 6: Statistical Analysis
#### Input Files
- `dataB.csv`, `length_data_1.csv`, `length_data_2n.csv`, `length_data_3.csv`: CSV files for statistical analysis.

#### Code Files
- `box_plots.py`: Script for generating box plots.
- `plot_scatter_2D.py`: Script for creating scatter plots.

#### Output Files
- `scatter_plot2D.pdf`: Scatter plot visualization.
- `traffic_statistic.pdf`: Statistical report.

## Usage
1. Navigate to the desired chapter directory.
2. Use the input files to run the corresponding code scripts.
3. Outputs will be generated in the respective `3_Output` directories.


For any questions or clarifications, please contact Alfan Presekal (A.Presekal@tudelft.nl).
